#include "acdptu.h"

#include "dptu_c_types.h"
#include "dptu_driver.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>


static genom_event ptu_error(const char *error, genom_context self);

static FILE *sfile = NULL;

/* --- Task pos_pub ----------------------------------------------------- */

/* --- Activity monitor_position ---------------------------------------- */

/** Codel dptu_start_monitor_position of activity monitor_position.
 *
 * Triggered by dptu_start.
 * Yields to dptu_get.
 * Throws dptu_e_file.
 */
genom_event
dptu_start_monitor_position(dptu_ptu_axis axis,
                            dptu_ptu_unit unit, dptu_ids *ids,
                            const dptu_ptu_position *ptu_position,
                            genom_context self)
{
  ids->monitor = true;
  dptu_timed_pos *pos = ptu_position->data(self);
  pos->id = 0;
  pos->axis = axis;
  pos->unit = unit;
  ptu_position->write(self);

  if (ids->save_type == dptu_ENABLED) {
    sfile = fopen(ids->save_file, "a");
    if (sfile == NULL)
      perror("Unable to open save file: ");
  }

  /* skeleton sample */ return dptu_get;
}

/** Codel dptu_get_monitor_position of activity monitor_position.
 *
 * Triggered by dptu_get.
 * Yields to dptu_get, dptu_ether.
 * Throws dptu_e_file.
 */
genom_event
dptu_get_monitor_position(dptu_ptu_axis axis, dptu_ptu_unit unit,
                          dptu_ids *ids,
                          const dptu_ptu_position *ptu_position,
                          genom_context self)
{
  dptu_t *p = (dptu_t *)ids->ptu;
  if (p == NULL) {
    return ptu_error("PTU not initialised", self);
  }

  const char *cmd;
  if (axis == dptu_PAN) {
    cmd = CMD_ABS_PAN_POS;
  } else { // dptu_TILT
    cmd = CMD_ABS_TILT_POS;
  }

  double value;
  long lvalue;
  struct timespec time_stamp;

  if (ptuCmdGetLongT(p, cmd, &lvalue, &time_stamp) < 0) {
    return ptu_error("Unable to send ptu set command", self);
  }

  if (unit == dptu_STEP) {
    value = (double)lvalue;
  } else if (unit == dptu_DEG) {
    value = PT_STEP_TO_DEG(p, lvalue);
  } else { // dptu_RAD
    value = PT_STEP_TO_RAD(p, lvalue);
  }

  dptu_timed_pos *pos = ptu_position->data(self);
  pos->id++;
  pos->time_stamp.sec = time_stamp.tv_sec;
  pos->time_stamp.nsec = time_stamp.tv_nsec;
  pos->value = value;

  ptu_position->write(self);

  if (sfile != NULL) {
    fprintf(sfile, "%lf, %lf\n", ((double)time_stamp.tv_sec) * 1e3 +
                                     ((double)time_stamp.tv_nsec) * 1e-6,
            value);
  }

  if (!ids->monitor) {
    if (sfile != NULL) fclose(sfile);
    return dptu_ether;
  }
  /* skeleton sample */ return dptu_get;
}

static genom_event ptu_error(const char *error, genom_context self) {
  dptu_e_ptu_detail d;
  strcpy(d.what, error);
  return dptu_e_ptu(&d, self);
}
